<?php

    namespace classifier\controllers;

    use classifier\helpers\CommandHelper;
    use Yii;
    use yii\db\ActiveRecord;

    use yii\helpers\Inflector;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    use yii\filters\VerbFilter;

    /**
     * CommonController implements the CRUD actions for all classifiers.
     */
    class CommonController extends Controller
    {
        /**
         * Modelul Search
         * @var
         */
        public $SearchModel;
        /**
         * stocheaza modelul curent a clasificatorului
         * @var ActiveRecord
         */
        public $Model;
        /**
         * stocheaza denumirea modelului curent impreuna cu nume de spatiu
         * @var string
         */
        public $ModelName;

        /**
         * FUnctie destinata initializarii parametrilor in dependenta de modelul curent
         * @param $action
         * @return bool
         */
        public function beforeAction($action)
        {
            $this->ModelName = CommandHelper::DEFAULT_NS . "\\" . Inflector::id2camel(Yii::$app->controller->id, '-');
            $modelName = $this->ModelName;
            $this->Model = new $modelName;
            $searchMN = $modelName . "Search";
            $this->SearchModel = new $searchMN;
            return true;
        }

        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ];
        }

        /**
         * Actiunea Index
         * @return string
         */
        public function actionIndex()
        {
            $searchModel = $this->SearchModel;
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);
            $gridColumns = [];
            $gridColumns[] = ['class' => 'yii\grid\SerialColumn'];
            $modelName = $this->ModelName;
            $gridColumns = array_merge($gridColumns, $this->Model->attributes());
            $gridColumns[] = ['class' => 'yii\grid\ActionColumn'];
            return $this->render('@classifier/views/classifier/index', [
                'searchModel' => $searchModel,
                'dataProvider' => $dataProvider,
                'gridColumns' => $gridColumns,
                'modelName' => $this->ModelName
            ]);
        }

        /**
         * @param $id
         * @return string
         * @throws NotFoundHttpException
         */
        /*@todo primery key duble de realizat view*/
        public function actionView($id)
        {
            return $this->render('@classifier/views/classifier/view', [
                'model' => $this->findModel($id),
                'modelName' => $this->ModelName
            ]);
        }

        /**
         * Creates a new TestTable12 model.
         * If creation is successful, the browser will be redirected to the 'view' page.
         * @return mixed
         */
        public function actionCreate()
        {

            $model = $this->Model;

            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }

            return $this->render('@classifier/views/classifier/create', [
                'model' => $model,
                'modelName' => $this->ModelName
            ]);
        }

        /**
         * Actiune folosita pentru crearea unui row in clasificator pe baza de request Ajax
         * folosita pentru addon in Select2
         * @return array|string
         */
        public function actionCreateAjax()
        {

            $model = $this->Model;
            if (!empty($_GET["label"])) {
                $label = $_GET["label"];
            } else {
                $label = $model->primaryKey;
            }
            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return json_encode(['id' => $model->primaryKey, 'label' => $model->$label]);
            }

            return $this->renderAjax('@classifier/views/classifier/_form', [
                'model' => $model,
                "modelName" => $this->ModelName
            ]);

        }

        /**
         * Actiune de actualizare a modeluluio curent
         * @param $id
         * @return string|\yii\web\Response
         * @throws NotFoundHttpException
         */
        public function actionUpdate($id)
        {
            $model = $this->findModel($id);

            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id, 'name' => $model->name]);
            }

            return $this->render('@classifier/views/classifier/update', [
                'model' => $model,
                'modelName' => $this->ModelName
            ]);
        }

        /**
         *  Actiune de stergere a modelului curent
         * @param $id
         * @return \yii\web\Response
         * @throws NotFoundHttpException
         */
        public function actionDelete($id)
        {
            $this->findModel($id)->delete();

            return $this->redirect(['index']);
        }

        /*@todo primery key duble de realizat view*/
        protected function findModel($id)
        {
            $modelName = $this->ModelName;
            if (($model = $modelName::findOne(['id' => $id])) !== null) {
                return $model;
            }

            throw new NotFoundHttpException(Yii::t('classifier', 'The requested page does not exist.'));
        }
    }
