<?php

    namespace classifier\controllers;

    use classifier\helpers\CommandHelper;
    use classifier\generators\model\GeneratorHelper;

    use classifier\models\SysClassifiers;


    use yii\elasticsearch\Query;
    use yii\helpers\Url;
    use yii\web\Controller;

    /**
     * Default controller for the `Classifier` module
     */
    class DefaultController extends Controller
    {
        /**
         * Renders the index view for the module
         * @return string
         */
        public function actionIndex()
        {

            return $this->render('index');
        }


        /**
         * action called with GET method, destinate to
         * -generate Model
         * -generate CRUD
         * @param $tableName
         * @param bool $crud
         * @param bool $model
         * @param string $relationsLabel
         * @return string|\yii\web\Response
         */
        public function actionGenerateModel($tableName, $crud = false, $model = true,$relationsLabel="{}")
        {

            $relationsLabel=json_decode($relationsLabel,true);
            $result = ["state" => true, "results" => null];
            $generator = new GeneratorHelper();
            $className = $generator->getClassName($tableName);
            $classNameNS = CommandHelper::DEFAULT_NS . "\\" . $className;

            if ($model == true) {
                $data = [
                    "tableName" => CommandHelper::DEFAULT_SCHEMA . "." . $tableName,
                    "ns" => CommandHelper::DEFAULT_NS,
                    "searchModelClass" => $classNameNS . "Search",
                    "relationsLabel"=>$relationsLabel,
                    "baseClass" => "yii\db\ActiveRecord",
                    "db" => 'db',
                    "useTablePrefix" => 0,
                    "generateRelations" => "all",
                    "generateRelationsFromCurrentSchema" => 0,
                    "generateLabelsFromComments" => 1,
                    "generateElastic" => 1,
                    "generateQuery" => 0,
                    "queryNs" => 'app\models',
                    "queryBaseClass" => 'yii\db\ActiveQuery',
                    "enableI18N" => 1,
                    "messageCategory" => 'app',
                    "useSchemaName" => 0,
                    "template" => "default"
                ];
                $classifier = SysClassifiers::findOne(["table_name" => $tableName]);
                $result = $generator->generateModel($data);
                $classifier->class_name = $classNameNS;
                if (class_exists(($classNameNS . "Elastic"))) {
                    try {
                        if (\classifier\helpers\CompatibilityHelper::isCompatibleElastic()) {
                            ($classNameNS . "Elastic")::deleteIndex();
                            ($classNameNS . "Elastic")::createIndex();
                        }
                    }catch (\Exception $e)
                    {
                        \Yii::$app->session->setFlash("error","Elastic Error");
                    }
                }
                $classifier->save();
            }
            else {
                if (!class_exists($classNameNS)) {
                    $data = [
                        "tableName" => CommandHelper::DEFAULT_SCHEMA . "." . $tableName,
                        "ns" => CommandHelper::DEFAULT_NS,
                        "searchModelClass" => $classNameNS . "Search",
                        "relationsLabel" => $relationsLabel,
                        "baseClass" => "yii\db\ActiveRecord",
                        "db" => 'db',
                        "useTablePrefix" => 0,
                        "generateRelations" => "all",
                        "generateRelationsFromCurrentSchema" => 0,
                        "generateLabelsFromComments" => 1,
                        "generateQuery" => 0,
                        "generateElastic" => 1,
                        "queryNs" => 'app\models',
                        "queryBaseClass" => 'yii\db\ActiveQuery',
                        "enableI18N" => 1,
                        "messageCategory" => 'app',
                        "useSchemaName" => 0,
                        "template" => "default"
                    ];

                    $result = $generator->generateElastic($data);
                    if (class_exists(($classNameNS . "Elastic"))) {
                        try {
                            if (\classifier\helpers\CompatibilityHelper::isCompatibleElastic()) {
                                ($classNameNS . "Elastic")::deleteIndex();
                                ($classNameNS . "Elastic")::createIndex();
                            }
                        }catch (\Exception $e)
                        {
                            \Yii::$app->session->setFlash("error","Elastic Error");
                        }
                    }
                }
            }
            if ($result["state"] && $crud) {
                $classifier = SysClassifiers::findOne(["table_name" => $tableName]);
                if (!empty($classifier)) {
                    $generator = new \classifier\generators\crud\GeneratorHelper();;
                    $data = [
                        "modelClass" => $classNameNS,
                        "searchModelClass" => null,
                        "controllerClass" => $generator->controllerNS . $className . "Controller",
                        "baseControllerClass" => "yii\web\Controller",
                        "indexWidgetType" => "grid",
                        "enableI18N" => 1,
                        "enablePjax" => 1,
                        "messageCategory" => $generator->messageCategory,
                        "template" => $generator->template,
                        "relationsLabel"=>$relationsLabel,
                    ];


                    $result = $generator->generateCrud($data);
                    if($result["state"])
                    {
                        return $this->redirect(Url::to(["/Classifier/sys-classifiers/view","id"=>$classifier->id]));
                    }
                }
            } elseif (!$crud) {
                $classifier = SysClassifiers::findOne(["table_name" => $tableName]);
                if (!empty($classifier)) {

                    $generator = new \classifier\generators\controller\GeneratorHelper();
                    $data = [
                        "controllerClass" => $generator->controllerNS . $className . "Controller",
                        "actions" => null,
                        "viewPath" => null,
                        "template" => $generator->template
                    ];
                    $result = $generator->generateController($data);
                    if($result["state"])
                    {
                        return $this->redirect(Url::to(["/Classifier/sys-classifiers/view","id"=>$classifier->id]));
                    }
                }

            }
            /*@todo de returnat eroare*/
            return json_encode($result);

        }




        public function actionRegenerateModel($classifier, $tabel = false, $crud = false, $model = true,$relationsLabel="{}")
        {
            $classifier = SysClassifiers::findOne($classifier);
            $classifierData["ClassifierForm"] = [
                "tableName" => $classifier->table_name,
                "generateCrud" => $crud,
                "generateTable" => $tabel,
                "generateModel" => $model,
                "relationsLabel"=>$relationsLabel,
                "fields" => json_decode($classifier->attribute_fields, true),
                "description" => $classifier->description,
            ];
            $response = CommandHelper::ClassifierCreate($classifierData);
            $response = json_decode($response, true);
            if (!$response["error"]) {
                return $this->redirect($response["response"]);
            } else {
                die(var_dump($response));
            }
        }
        public function actionCreateIndex($classifier)
        {
            $classifier = SysClassifiers::findOne($classifier);
            $clasifierElasticClass = $classifier->class_name . "Elastic";
            $clasifierElasticClass::createIndex();
            return $this->redirect(Url::to(["/Classifier/sys-classifiers/view", "id" => $classifier->id]));
        }

        public function actionCreateElastic($classifier)
        {
            $classifier = SysClassifiers::findOne($classifier);
            $classifierData["RegisterForm"] = [
                "tableName" => $classifier->table_name,
                "generateCrud" => false,
                "generateTable" => false,
                "generateModel" => false,
                "fields" => json_decode($classifier->attribute_fields, true),
                "description" => $classifier->description,
            ];
            $response = CommandHelper::ClassifierCreate($classifierData);
            $response = json_decode($response, true);
            if (!$response["error"]) {
                return $this->redirect($response["response"]);
            } else {
                die(var_dump($response));
            }
        }

        public function actionSearch($search)
        {

            $query = new Query();
            $query->highlight([
                "pre_tags" => ["<span style='background-color: rgba(0,204,102,0.47)'>"],
                "post_tags" => ["</span>"],
                "fields" => ["_all" => ["pre_tags" => ["<em>"], "post_tags" => ["</em>"]]],
                "fragment_size" => 2147483647
            ]);
            $query->query = ["query_string" => ["query" => $search]];


            die(var_dump($query->all()));

        }


    }
