<?php
    /**
     * Created by PhpStorm.
     * User: startreksi
     * Date: 5/11/18
     * Time: 12:12 PM
     */

    namespace classifier\controllers;


    use classifier\helpers\CommandHelper;
    use Yii;
    use yii\db\Connection;
    use yii\di\Instance;

    class MigrateController extends \yii\console\controllers\MigrateController
    {
        public $labels=[];//comentarii pentru coloane
        /**
         * {@inheritdoc}
         * @since 2.0.8
         */
        public function options($actionID)
        {
            return array_merge(
                parent::options($actionID),
                ['migrationTable', 'db'], // global for all actions
                $actionID === 'create'
                    ? ['templateFile', 'fields', 'useTablePrefix', 'comment','labels']
                    : []
            );
        }

        /**
         * Generearea migrarii
         * @param $params
         * @return string
         * @throws \yii\base\InvalidConfigException
         */
        protected function generateMigrationSourceCode($params)
        {
            $parsedFields = $this->parseFields();
            $fields = $parsedFields['fields'];

            $foreignKeys = $parsedFields['foreignKeys'];
            $primaryKey=[];

            $name = $params['name'];

            $templateFile = $this->templateFile;
            $table = null;
            if (preg_match('/^create_junction(?:_table_for_|_for_|_)(.+)_and_(.+)_tables?$/', $name, $matches)) {
                $templateFile = $this->generatorTemplateFiles['create_junction'];
                $firstTable = $matches[1];
                $secondTable = $matches[2];

                $fields = array_merge(
                    [
                        [
                            'property' => $firstTable . '_id',
                            'decorators' => 'integer()',
                        ],
                        [
                            'property' => $secondTable . '_id',
                            'decorators' => 'integer()',
                        ],
                    ],
                    $fields,
                    [
                        [
                            'property' => 'PRIMARY KEY(' .
                                $firstTable . '_id, ' .
                                $secondTable . '_id)',
                        ],
                    ]
                );

                $foreignKeys[$firstTable . '_id']['table'] = $firstTable;
                $foreignKeys[$secondTable . '_id']['table'] = $secondTable;
                $foreignKeys[$firstTable . '_id']['column'] = null;
                $foreignKeys[$secondTable . '_id']['column'] = null;
                $table = $firstTable . '_' . $secondTable;
            } elseif (preg_match('/^add_(.+)_columns?_to_(.+)_table$/', $name, $matches)) {
                $templateFile = $this->generatorTemplateFiles['add_column'];
                $table = $matches[2];
            } elseif (preg_match('/^drop_(.+)_columns?_from_(.+)_table$/', $name, $matches)) {
                $templateFile = $this->generatorTemplateFiles['drop_column'];
                $table = $matches[2];
            } elseif (preg_match('/^create_(.+)_table$/', $name, $matches)) {
                if(empty($parsedFields["primaryKey"])) {
                    $this->addDefaultPrimaryKey($fields);
                }
                $templateFile = $this->generatorTemplateFiles['create_table'];
                $schema=CommandHelper::DEFAULT_SCHEMA;
                $table = $matches[1];
            } elseif (preg_match('/^drop_(.+)_table$/', $name, $matches)) {
                $this->addDefaultPrimaryKey($fields);
                $templateFile = $this->generatorTemplateFiles['drop_table'];
                $table = $matches[1];
            }

            foreach ($foreignKeys as $column => $foreignKey) {

                $relatedColumn = $foreignKey['column'];
                $relatedTable = $foreignKey['table'];
                // Since 2.0.11 if related column name is not specified,
                // we're trying to get it from table schema
                // @see https://github.com/yiisoft/yii2/issues/12748
                if ($relatedColumn === null) {
                    $relatedColumn = 'id';
                    try {
                        $this->db = Instance::ensure($this->db, Connection::className());
                        $relatedTableSchema = $this->db->getTableSchema($relatedTable);
                        if ($relatedTableSchema !== null) {
                            $primaryKeyCount = count($relatedTableSchema->primaryKey);
                            if ($primaryKeyCount === 1) {
                                $relatedColumn = $relatedTableSchema->primaryKey[0];
                            } elseif ($primaryKeyCount > 1) {
                                $this->stdout("Related table for field \"{$column}\" exists, but primary key is composite. Default name \"id\" will be used for related field\n", Console::FG_YELLOW);
                            } elseif ($primaryKeyCount === 0) {
                                $this->stdout("Related table for field \"{$column}\" exists, but does not have a primary key. Default name \"id\" will be used for related field.\n", Console::FG_YELLOW);
                            }
                        }
                    } catch (\ReflectionException $e) {
                        $this->stdout("Cannot initialize database component to try reading referenced table schema for field \"{$column}\". Default name \"id\" will be used for related field.\n", Console::FG_YELLOW);
                    }
                }

                $foreignKeys[$column] = [
                    'idx' => $this->generateTableName("idx-$table-$column"),
                    'fk' => $this->generateTableName("fk-$table-$column"),
                    'relatedTable' => $this->generateTableName($relatedTable),
                    'relatedColumn' => $relatedColumn,
                ];
            }
            if(!empty($parsedFields["primaryKey"]))
            {
                $primaryKey=$parsedFields["primaryKey"];
                $pk_columns=implode("_",$primaryKey["columns"]);
                $primaryKey["pk"]=$this->generateTableName("pk-$table-$pk_columns");
            }

            return $this->renderFile(Yii::getAlias($templateFile), array_merge($params, [
                'table' =>$schema.".". $this->generateTableName($table),
                'fields' => $fields,
                'foreignKeys' => $foreignKeys,
                'primaryKey' => $primaryKey,
                'tableComment' => $this->comment,
            ]));
        }

        /**
         * Formarea atributelor
         * @return array
         */
        protected function parseFields()
        {
            $fields = [];
            $foreignKeys = [];
            $primaryKey = [];

            foreach ($this->fields as $index => $field) {
                $chunks = preg_split('/\s?:\s?/', $field, null);
                $property = array_shift($chunks);
                if ($property === 'primaryKey') {
                    $primaryKey = [
                        "pk"=>"",
                        "columns" => $chunks
                    ];
                } else {
                    foreach ($chunks as $i => &$chunk) {
                        if (strncmp($chunk, 'foreignKey', 10) === 0) {
                            preg_match('/foreignKey\((\w*)\s?(\w*)\)/', $chunk, $matches);

                            $foreignKeys[$property] = [
                                'table' => isset($matches[1])
                                    ? CommandHelper::DEFAULT_SCHEMA.".".$matches[1]
                                    : preg_replace('/_id$/', '', $property),
                                'column' => !empty($matches[2])
                                    ? $matches[2]
                                    : null,
                            ];

                            unset($chunks[$i]);
                            continue;
                        }

                        if (!preg_match('/^(.+)\)$/',$chunk)&&!preg_match('/^(.+?)\(([^(]+)\)$/', $chunk)) {
                            $chunk .= '()';
                        }
                    }
                    $fields[] = [
                        'property' => $property,
                        'decorators' => implode('->', $chunks),
                        'comment'=>""
                    ];
                }
            }
            $labels=[];

            foreach ($this->labels as $index => $label) {
                $chunks = preg_split('/\s?:\s?/', $label, null);
                $property = array_shift($chunks);
                if ($property === 'primaryKey') {
                    $primaryKey = [
                        "pk"=>"",
                        "columns" => $chunks
                    ];
                } else {
                    $labels[$property] = $chunks[0];
                }
            }
            foreach ($fields as $key=>$field)
            {
                $fields[$key]["comment"]=$labels[$field["property"]];
            }

            return [
                'fields' => $fields,
                'foreignKeys' => $foreignKeys,
                'primaryKey' => $primaryKey,
            ];
        }

    }