<?php

    namespace classifier\controllers;

    use classifier\helpers\CommandHelper;
    use Yii;
    use classifier\models\SysClassifiers;
    use classifier\models\SysClassifiersSearch;


    use yii\helpers\Inflector;
    use yii\web\Controller;
    use yii\web\NotFoundHttpException;
    use yii\filters\VerbFilter;

    /**
     * SysClassifiersController implements the CRUD actions for SysClassifiers model.
     */
    class SysClassifiersController extends Controller
    {
        /**
         * @inheritdoc
         */
        public function behaviors()
        {
            return [
                'verbs' => [
                    'class' => VerbFilter::className(),
                    'actions' => [
                        'delete' => ['POST'],
                    ],
                ],
            ];
        }

        /**
         * Lists all SysClassifiers models.
         * @return mixed
         */
        public function actionIndex()
        {
            $searchModel = new SysClassifiersSearch();
            $dataProvider = $searchModel->search(Yii::$app->request->queryParams);

            return $this->render('index', [
                'searchModel' => $searchModel,
                'dataProvider' => $dataProvider,
            ]);
        }

        /**
         * Displays a single SysClassifiers model.
         * @param integer $id
         * @return mixed
         * @throws NotFoundHttpException if the model cannot be found
         */
        public function actionView($id)
        {
            return $this->render('view', [
                'model' => $this->findModel($id),
            ]);
        }

        /**
         * Creates a new SysClassifiers model.
         * If creation is successful, the browser will be redirected to the 'view' page.
         * @return mixed
         */
        public function actionCreate()
        {
            return $this->redirect(['/classifiercreator/default/classificatory']);
           /* $model = new SysClassifiers();

            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }

            return $this->render('create', [
                'model' => $model,
            ]);*/
        }

        /**
         * Updates an existing SysClassifiers model.
         * If update is successful, the browser will be redirected to the 'view' page.
         * @param integer $id
         * @return mixed
         * @throws NotFoundHttpException if the model cannot be found
         */
        public function actionUpdate($id)
        {
            $model = $this->findModel($id);

            if ($model->load(Yii::$app->request->post()) && $model->save()) {
                return $this->redirect(['view', 'id' => $model->id]);
            }

            return $this->render('update', [
                'model' => $model,
            ]);
        }

        /**
         * * Deletes an existing SysClassifiers model.
         * If deletion is successful, the browser will be redirected to the 'index' page.
         * @param $id
         * @return \yii\web\Response
         * @throws NotFoundHttpException
         * @throws \Exception
         * @throws \Throwable
         * @throws \yii\db\Exception
         * @throws \yii\db\StaleObjectException
         */
        public function actionDelete($id)
        {
            $model = $this->findModel($id);
            $tableName = CommandHelper::DEFAULT_SCHEMA . "." . $model->table_name;

            if (!is_null(Yii::$app->db->schema->getTableSchema($tableName, true))) {
                $command = Yii::$app->db->createCommand();
                $command->dropTable($tableName);
                $sql=$command->getSql();
                $command->setSql($sql." CASCADE");
                $command->execute();

            }
            $className = $model->class_name;
            $elasticClass=$model->class_name."Elastic";
            if(class_exists($elasticClass))
            {
                if(\classifier\helpers\AppHelper::getConfigs("ELASTIC")) {
                    $elasticClass::deleteIndex();
                }
            }
            $className = \yii\helpers\StringHelper::basename($className);
            $elasticClassName = \yii\helpers\StringHelper::basename($elasticClass);

            $modelPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "models" . DIRECTORY_SEPARATOR . $className . ".php";
            $modelElasticPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "models" . DIRECTORY_SEPARATOR . $elasticClassName . ".php";
            $modelSearchPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "models" . DIRECTORY_SEPARATOR . $className . "Search.php";
            $controllerPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "controllers" . DIRECTORY_SEPARATOR . $className . "Controller.php";
            $viewsPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "views" . DIRECTORY_SEPARATOR . Inflector::camel2id($className);
            $migrationsPath = Yii::getAlias("@console") . DIRECTORY_SEPARATOR . "migrations" . DIRECTORY_SEPARATOR . $model->table_name;
            if (is_dir($viewsPath)) {
                CommandHelper::rrmdir($viewsPath);
            }
            if (is_dir($migrationsPath)) {
                CommandHelper::rrmdir($migrationsPath);
            }
            if (is_file($modelPath)) {
                unlink($modelPath);
            }
            if (is_file($modelElasticPath)) {
                unlink($modelElasticPath);}
            if (is_file($modelSearchPath)) {
                unlink($modelSearchPath);
            }
            if (is_file($controllerPath)) {
                unlink($controllerPath);
            }
            $model->delete();

            return $this->redirect(['index']);


        }

        /**
         * Finds the SysClassifiers model based on its primary key value.
         * If the model is not found, a 404 HTTP exception will be thrown.
         * @param integer $id
         * @return SysClassifiers the loaded model
         * @throws NotFoundHttpException if the model cannot be found
         */
        protected function findModel($id)
        {
            if (($model = SysClassifiers::findOne($id)) !== null) {
                return $model;
            }

            throw new NotFoundHttpException(Yii::t('classifiers', 'The requested page does not exist.'));
        }
    }
