<?php

    namespace classifier\generators\crud;


    use classifier\helpers\CommandHelper;
    use Yii;


    use yii\helpers\BaseHtml;
    use yii\helpers\Inflector;
    use yii\helpers\VarDumper;


    class GeneratorHelper extends yii\gii\generators\crud\Generator
    {
        public $relationsLabel="id";
        public $baseControllerClass = 'classifier\controllers\CommonController';
        /**
         * template name for CRUD
         * @var string
         */
        public $template="default";
        /**
         * message Category for models and Search models
         * @var string
         */
        public $messageCategory="classifier";
        /**
         * default Name Space for controllers in classifier
         * @var string
         */
        public $controllerNS="classifier\\controllers\\";

        /**
         * Generate Crud for model by data in
         * @param $data
         * @return array
         */
        public function generateCrud($data)
        {
            $this->loadStickyAttributes();
            $this->attributes = $data;
            $this->relationsLabel=$data["relationsLabel"];
            $files = $this->generate();

            $answers = [];
            foreach ($files as $file) {
                $answers[$file->id] = $file->id;
            }
            $results = "";
            $isError = !$this->save($files, $answers, $results);

            return ["state"=>!$isError,"results"=>$results];


        }

        /**
         * rewrited method for implemet @classifier allias in geting view path for CRUD
         * @return bool|string
         */
        public function getViewPath()
        {
            if (empty($this->viewPath)) {
                return Yii::getAlias('@classifier'.DIRECTORY_SEPARATOR.'views'.DIRECTORY_SEPARATOR . $this->getControllerID());
            }

            return Yii::getAlias(str_replace('\\', '/', $this->viewPath));
        }
        public  function  isForeignColumn($attribute)
        {
            $tableSchema = $this->getTableSchema();
            foreach ($tableSchema->foreignKeys as $foreignKey)
            {

                if(!empty($foreignKey[$attribute]))
                {
                    return true;
                }
            }
            return false;
        }
        public function generateActiveField($attribute)
        {
            //@todo de scos numele tabelului si deimplementat exemplu
            $tableSchema = $this->getTableSchema();
            $AttributeName=BaseHtml::getInputId(new $this->modelClass,$attribute);

            if ($tableSchema === false || !isset($tableSchema->columns[$attribute])) {
                if (preg_match('/^(password|pass|passwd|passcode)$/i', $attribute)) {
                    return "\$form->field(\$model, '$attribute')->passwordInput()";
                }

                return "\$form->field(\$model, '$attribute')";
            }
            $column = $tableSchema->columns[$attribute];

            if($this->isForeignColumn($attribute))
            {
                $relationLabel=$this->relationsLabel;
                foreach ($tableSchema->foreignKeys as $foreignKey)
                {

                    if(!empty($foreignKey[$attribute]))
                    {
                        $relationTabel=$foreignKey[0];
                        $relationTabel=str_replace(CommandHelper::DEFAULT_SCHEMA.".","",$relationTabel);
                        $relationLabel=$this->relationsLabel[$relationTabel];
                        $controller=Inflector::camel2id($relationTabel);
                        break;
                    }
                }
                $attributeMethodName="getForeignData".Inflector::camel2id($attribute,'_');
               return "\$form->field(\$model, '$attribute')->widget('\\kartik\\widgets\\Select2', [
                    'data' => \$model->$attributeMethodName(),
                    'addon' => [
                        'prepend' => [
                            'content' =>Html::button('<i class=\"fa fa-plus\"></i>', ['id' => 'classifier_".$attribute."_btn_sent', 'value' => \yii\helpers\Url::to(['/Classifier/$controller/create-ajax']), 'class' => 'btn btn-success open_classifier_modal']) ,
                             'asButton' => true,
                        ],
                    ],
                        'options' => ['placeholder' => Yii::t('classifier','$attribute'), 'class' => 'form-control'],
                        'pluginOptions' => [
                            'allowClear' => true
                        ],
                   ]);
                   ?>
                  <?php
        Pjax::begin();
        \$script = <<< JS
        function isJson(str) {
            try {
                JSON.parse(str);
            } catch (e) {
                return false;
            }
            return true;
        }
        function initAjaxForm_$attribute() {
           $('#modal_classifier_$attribute form').on('submit', function (e)  {
                         e.preventDefault();
                         e.stopImmediatePropagation();
                         e.stopPropagation();
                        var form =$(this);
                        
                        jQuery.ajax({
                            url: '/Classifier/$controller/create-ajax?label=$relationLabel',
                            type: 'post',
                            complete:initAjaxForm_$attribute,
                            data: form.serialize(),     
                            success: function (data) {
                              
                                if(!isJson(data))
                                    {
                                     $('#modal_classifier_$attribute').modal('show')
                                            .find('.add_modal_form_classifier')
                                            .html(data)
                                             
                                 }else 
                                 {
                                    $('#modal_classifier_$attribute') .find('.add_modal_form_classifier').html('')
                                     data=JSON.parse(data)
                                     var newOption = new Option(data.label, data.id, true, true);
                                     $('#$AttributeName').append(newOption).trigger('change');
                                     $('#modal_classifier_$attribute').modal('hide')
                                 }
                            },
                         
                        });
                        return false;
                    });
        }
                   $(\"#modal_classifier_$attribute\").on(\"shown.bs.modal\",function() {
                       
                     initAjaxForm_$attribute();
                   })
                    $(function () {
                        $('#classifier_".$attribute."_btn_sent').click(function () {
                            $('#modal_classifier_$attribute').modal('show')
                                .find('.add_modal_form_classifier')
                                .load($(this).attr('value'))
                                
                        });
                    });
JS;
        \$this->registerJs(\$script);
        Modal::begin([
            'header' => '<h4>'.Yii::t('classifier','Create new Item').'</h4>',
            'id' => 'modal_classifier_$attribute',
            'size' => 'model-lg',
        ]);
        Pjax::begin();
    ?>
    <div class=\"error_classifier_create\"></div>
    <div class=\"add_modal_form_classifier\"></div>
    <?php
        Pjax::end();
        Modal::end();
        Pjax::end();
  
                   
                   ";
            }
            if ($column->phpType === 'boolean') {
                return "\$form->field(\$model, '$attribute')->checkbox()";
            }

            if ($column->type === 'text') {
                return "\$form->field(\$model, '$attribute')->textarea(['rows' => 6])";
            }

            if (preg_match('/^(password|pass|passwd|passcode)$/i', $column->name)) {
                $input = 'passwordInput';
            } else {
                $input = 'textInput';
            }
            if ($column->phpType === 'integer') {
                return "\$form->field(\$model, '$attribute')->widget('\\kartik\\number\\NumberControl', [
                \"name\" => \$model,
                'maskedInputOptions' => ['digits' => 0],
            ]);";
            }
            if ($column->type === 'date') {
                return "\$form->field(\$model, '$attribute')->widget(\"\\kartik\\datecontrol\\Datecontrol\", [
                'type' => \\kartik\\datecontrol\\DateControl::FORMAT_DATE,
                'ajaxConversion' => false,
                'widgetOptions' => [
                    'pluginOptions' => [
                        'autoclose' => true
                    ]
                ]
            ]);";
            }
            if ($column->type === 'timestamp') {
                return "\$form->field(\$model, '$attribute')->widget(\"\\kartik\\datecontrol\\Datecontrol\", [
                'type' => \\kartik\\datecontrol\\DateControl::FORMAT_DATETIME,
                'ajaxConversion' => false,
                'widgetOptions' => [
                    'pluginOptions' => [
                        'autoclose' => true
                    ]
                ]
            ]);";
            }

            if ($column->type === 'time') {
                return "\$form->field(\$model, '$attribute')->widget(\"\\kartik\\datecontrol\\Datecontrol\", [
                'type' => \\kartik\\datecontrol\\DateControl::FORMAT_TIME,
                'ajaxConversion' => false,
                'widgetOptions' => [
                    'pluginOptions' => [
                        'autoclose' => true
                    ]
                ]
            ]);";
            }

            if (is_array($column->enumValues) && count($column->enumValues) > 0) {
                $dropDownOptions = [];
                foreach ($column->enumValues as $enumValue) {
                    $dropDownOptions[$enumValue] = Inflector::humanize($enumValue);
                }
                return "\$form->field(\$model, '$attribute')->dropDownList("
                    . preg_replace("/\n\s*/", ' ', VarDumper::export($dropDownOptions)).", ['prompt' => ''])";
            }

            if ($column->phpType !== 'string' || $column->size === null) {
                return "\$form->field(\$model, '$attribute')->$input()";
            }

            return "\$form->field(\$model, '$attribute')->$input(['maxlength' => true])";
        }
    }
