<?php
    /**
     * This is the template for generating the model class of a specified table.
     */

    use yii\helpers\Inflector;

    /* @var $this yii\web\View */
    /* @var $generator yii\gii\generators\model\Generator */
    /* @var $tableName string full table name */
    /* @var $className string class name */
    /* @var $queryClassName string query class name */
    /* @var $tableSchema yii\db\TableSchema */
    /* @var $properties array list of properties (property => [type, name. comment]) */
    /* @var $labels string[] list of attribute labels (name => label) */
    /* @var $rules string[] list of validation rules */
    /* @var $relations array list of relations (name => relation declaration) */

    echo "<?php\n";
?>

namespace <?= $generator->ns ?>;

use Yii;
use yii\helpers\ArrayHelper;
use yii\elasticsearch\ActiveDataProvider;
use yii\elasticsearch\Query;
/**
* This is the model class for table "<?= $generator->generateTableName($tableName) ?>".
*
<?php foreach ($properties as $property => $data): ?>
    * @property <?= "{$data['type']} \${$property}"  . ($data['comment'] ? ' ' . strtr($data['comment'], ["\n" => ' ']) : '') . "\n" ?>
<?php endforeach; ?>
<?php if (!empty($relations)): ?>
    *
    <?php foreach ($relations as $name => $relation): ?>
        * @property <?= $relation[1] . ($relation[2] ? '[]' : '') . ' $' . lcfirst($name) . "\n" ?>
    <?php endforeach; ?>
<?php endif; ?>
*/
class <?= $className ?> extends <?= '\\' . ltrim($generator->elasticClass, '\\') . "\n" ?>
{


/**
* @inheritdoc
*/
public function rules()
{
return [<?= empty($rules) ? '' : ("\n            " . implode(",\n            ", $rules) . ",\n        ") ?>];
}


<?php foreach ($relations as $name => $relation):
    $attributeMethodName="getForeignData".Inflector::camel2id($relation[3],'_');
    if(!empty($relation[5]))
    {
        $attributeMethodName="getForeignData".Inflector::camel2id($relation[5],'_');
    }

    ?>

    /**
    * @return \yii\db\ActiveQuery
    */
    public function get<?= $name ?>()
    {
    <?= $relation[0] . "\n" ?>
    }
    public function <?=$attributeMethodName?>()
    {
    $className='\registers\models\<?=$relation[1]?>';
    $data=ArrayHelper::map($className::find()->all(),$className::primaryKey(),'<?=$relation[3]?>');
    return $data;
    }
<?php endforeach; ?>
<?php if ($queryClassName): ?>
    <?php
    $queryClassFullName = ($generator->ns === $generator->queryNs) ? $queryClassName : '\\' . $generator->queryNs . '\\' . $queryClassName;
    echo "\n";
    ?>
    /**
    * @inheritdoc
    * @return <?= $queryClassFullName ?> the active query used by this AR class.
    */
    public static function find()
    {
    return new <?= $queryClassFullName ?>(get_called_class());
    }
<?php endif; ?>
    /**
    * @return array the list of attributes for this record
    */
    public function attributes()
    {
        return [
    <?php foreach ($phpTypePropreties as $property => $data): ?>

            '<?=$property?>',
    <?php
        endforeach;
    ?>
      ];
    }
    /**
    * @inheritdoc
    */
    public function attributeLabels()
    {
        return [
        <?php foreach ($labels as $name => $label): ?>
            <?= "'$name' => " . $generator->generateString($label) . ",\n" ?>
        <?php endforeach; ?>
        ];
    }

    /**
    * @return array This model's mapping
    */
    public static function mapping()
    {
        return [
            static::type() => [

                    'properties' => [
                        <?php foreach ($phpTypePropreties as $property => $data): ?>
                        <?php
                            if($data["type"]=="datetime"):
                                ?>
                                '<?=$property?>' => ['type' => 'date'],
                            <?php
                                        elseif($data["type"]=="integer"):
                                    ?>
'<?=$property?>' => ['type' => 'long'],
                                    <?php
                                    elseif($data["type"]!="text"):
                                    ?>
'<?=$property?>' => ['type' => '<?=$data["type"]?>'],
                                    <?php
                                    else:
                                    ?>
'<?=$property?>' => ['type' => '<?=$data["type"]?>', "analyzer" => "register_string_analyzer"],
                                    <?php
                                        endif;
                                     ?>
                            <?php
                                endforeach;
                            ?>

                            ]
                    ],
            ];
    }

    /**
    * Set (update) mappings for this model
    */
    public static function updateMapping()
    {
        $db = static::getDb();
        $command = $db->createCommand();
        $command->setMapping(static::index(), static::type(), static::mapping());
    }

    /**
    * Create this model's index
    */
    public static function createIndex()
    {
        $db = static::getDb();
        $command = $db->createCommand();
        $command->createIndex(static::index(), [
                    'settings' => [
                            "index" => [
                            "number_of_shards" => 1,
                            "number_of_replicas" => 0
                            ],
                            "analysis" => [
                            "filter" => [
                            "mynGram" => [
                            "type" => 'ngram',
                            "min_gram" => 3,
                            "max_gram" => 255
                            ]
                        ],
                        "analyzer" => [
                            "register_string_analyzer" => [
                            "type" => "custom",
                            "tokenizer" => 'standard',
                            "filter" => [
                            'mynGram'
                            ],
                        ]
                    ]
                ],
            ],

                    'mappings' => static::mapping(),
                    //'warmers' => [ /* ... */ ],
                    //'aliases' => [ /* ... */ ],
                    //'creation_date' => '...'
                    ]);
    }

    /**
    * Delete this model's index
    */
    public static function deleteIndex()
    {
        $db = static::getDb();
        $command = $db->createCommand();
        $command->deleteIndex(static::index(), static::type());
    }


    public function SearchesAll($data)
    {
        $search_query = null;
        $class = preg_replace("/(.*)[\W]/", "", self::class);

        if (!empty($data)) {

            foreach ($data[$class] as $key => $value) {
                $this->$key = $value;
                $search_query = $search_query . ($key == "id" ? "_id" : $key) . ":" . ($value == "" ? "*" : $value) . " && ";
            }
            $search_query = substr($search_query, 0, -3);
        }
        $search_query == null ? $search_query = "*" : $search_query;
        $query = new Query();
        $query->from(static::index());

        $query->query = [
            "query_string" => [
                // "fields" => $fields,
                "query" => $search_query
            ]
        ];


        $query->highlight(
            [
                'pre_tags' => ["<span style='background-color: yellow;'>"], // not required
                'post_tags' => ["</span>"], // not required
                "fields" => ["*" => new \stdClass()]
            ]);

        $dataProvider = new ActiveDataProvider([
            'query' => $query,
            'pagination' => ['pageSize' => 10],
        ]);

        return $dataProvider;
    }

}
