<?php
/**
 * This is the template for generating the model class of a specified table.
 */

    use yii\helpers\Inflector;

    /* @var $this yii\web\View */
/* @var $generator yii\gii\generators\model\Generator */
/* @var $tableName string full table name */
/* @var $className string class name */
/* @var $queryClassName string query class name */
/* @var $tableSchema yii\db\TableSchema */
/* @var $properties array list of properties (property => [type, name. comment]) */
/* @var $labels string[] list of attribute labels (name => label) */
/* @var $rules string[] list of validation rules */
/* @var $relations array list of relations (name => relation declaration) */

    $relationsLabel=[];
echo "<?php\n";
?>

namespace <?= $generator->ns ?>;

use Yii;
use yii\helpers\ArrayHelper;

/**
 * This is the model class for table "<?= $generator->generateTableName($tableName) ?>".
 *
<?php foreach ($properties as $property => $data): ?>
 * @property <?= "{$data['type']} \${$property}"  . ($data['comment'] ? ' ' . strtr($data['comment'], ["\n" => ' ']) : '') . "\n" ?>
<?php endforeach; ?>
<?php if (!empty($relations)): ?>
 *
<?php foreach ($relations as $name => $relation): ?>
 * @property <?= $relation[1] . ($relation[2] ? '[]' : '') . ' $' . lcfirst($name) . "\n" ?>
<?php endforeach; ?>
<?php endif; ?>
 */
class <?= $className ?> extends <?= '\\' . ltrim($generator->baseClass, '\\') . "\n" ?>
{
    /**
    * Functie apelata inainte de validarea clasificatorului
    * @param bool $insert
    * @return bool
    */
   public function beforeSave($insert) {

        if(empty($this->created_at)){
            $this->created_at =date("Y-m-d H:i:s");
        }
            $this->updated_at = date("Y-m-d H:i:s");
        if(empty($this->created_user))
        {
            $this->created_user=Yii::$app->user->id;
        }
        $this->updated_user=Yii::$app->user->id;
        return parent::beforeSave($insert);
    }
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return '<?= $generator->generateTableName($tableName) ?>';
    }
<?php if ($generator->db !== 'db'): ?>

    /**
     * @return \yii\db\Connection the database connection used by this AR class.
     */
    public static function getDb()
    {
        return Yii::$app->get('<?= $generator->db ?>');
    }
<?php endif; ?>

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [<?= empty($rules) ? '' : ("\n            " . implode(",\n            ", $rules) . ",\n        ") ?>];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
<?php foreach ($labels as $name => $label): ?>
            <?= "'$name' => " . $generator->generateString($label) . ",\n" ?>
<?php endforeach; ?>
        ];
    }

<?php foreach ($relations as $name => $relation):
    $attributeMethodName="getForeignData".Inflector::camel2id($relation[3],'_');
    if(!empty($relation[5]))
    {
        $attributeMethodName="getForeignData".Inflector::camel2id($relation[5],'_');
    }


    $relationsLabel[Inflector::camel2id($relation[1],'_')]=$relation[4];
    ?>

    /**
     * @return \yii\db\ActiveQuery
     */
    public function get<?= $name ?>()
    {
        <?= $relation[0] . "\n" ?>
    }
    public function <?=$attributeMethodName?>()
    {
        $className='\classifier\models\<?=$relation[1]?>';
        $data=ArrayHelper::map($className::find()->all(),$className::primaryKey(),'<?=$relation[4]?>');
        return $data;
    }
<?php endforeach; ?>
public $relationsLabel=[
<?php
    foreach ($relationsLabel as $table=>$relation):
?>
        "<?=$table?>"=>"<?=$relation?>",
<?php
    endforeach;
?>
];
<?php if ($queryClassName): ?>
<?php
    $queryClassFullName = ($generator->ns === $generator->queryNs) ? $queryClassName : '\\' . $generator->queryNs . '\\' . $queryClassName;
    echo "\n";
?>
    /**
     * @inheritdoc
     * @return <?= $queryClassFullName ?> the active query used by this AR class.
     */
    public static function find()
    {
        return new <?= $queryClassFullName ?>(get_called_class());
    }
<?php endif; ?>

<?php
    if($generator->generateElastic):
        $elasticClass=$generator->generateElasticClassName($className);
?>
    public function getElasticDocument()
    {
       if( \classifier\helpers\CompatibilityHelper::isCompatibleElastic())
        {
        $elasticModel=<?=$elasticClass?>::findOne($this->primaryKey);
        return $elasticModel;
        }
    }

    public function afterSave($insert, $changedAttributes)
    {
        $elastic=$this->getElasticDocument();
        if (empty($elastic)) {
            $elastic = new <?=$elasticClass?>();
            $elastic->primaryKey = $this->getPrimaryKey();
            $elastic->attributes=$this->attributes;
        } else {
            $elastic->attributes=$this->attributes;
        }
        try {
            if(\classifier\helpers\CompatibilityHelper::isCompatibleElastic())
            {
                $elastic->save();
            }
            } catch (Exception $exception) {
                return    Yii::error($exception->getMessage());
            }

        parent::afterSave($insert, $changedAttributes);
    }
    public function afterDelete()
    {

            $elastic=$this->getElasticDocument();
            if(!empty($elastic))
            $elastic->delete();

        parent::afterDelete();
    }



<?php
    endif;
?>

}
