<?php
    /**
     * Created by PhpStorm.
     * User: startreksi
     * Date: 5/3/18
     * Time: 2:48 PM
     */

    namespace classifier\helpers;


    use classifier\generators\model\GeneratorHelper;
    use classifier\models\ClassifierForm;
    use classifier\models\SysClassifiers;
    use Yii;

    use yii\console\Application;
    use yii\db\Query;
    use yii\helpers\ArrayHelper;

    /**
     * Helerul implicit ce contine functii pentru crearea comenzii de migrare dar si parametri constanti
     * Class CommandHelper
     * @package classifier\helpers
     */
    class CommandHelper
    {
        /**
         * Schema implicita unde se va stoca clasificatoarele
         */
        const DEFAULT_SCHEMA = "classifiers";
        /**
         * Namespace-ul implicit a clasificatoarelor
         */
        const DEFAULT_NS = "classifier\models";
        /**
         * sablonul de formare a migrarii
         */
        const CREATE_TEMPLATE = '
            migrate/create --fields="{@fields}"
        ';

        /**
         * Pe baza unui array de fiel-duri se genereaza comanda
         * @param array $fields
         * @return string
         */
        public static function CommandCreate($fields)
        {
            $fields = self::parseFields($fields);
            return strtr(self::CREATE_TEMPLATE,
                [
                    "{@fields}" => $fields,
                ]);
        }

        /**
         * Parsarea array-ului de fielduri intr-un string pentru comanda
         * Definirea cimpurilor de legaturiloe
         * Definirea etichetelor pentru clasificator
         * @param $fields
         * @return array
         */
        public static function parseFields($fields)
        {

            $field_string = "";
            $labels_string = "";
            $counter = 0;
            $relationColumns = [];
            foreach ($fields as $title => $field) {
                if ($counter != 0) {
                    $field_string .= ",";
                }
                if (is_array($field)) {
                    $field = (Object)$field;
                }
                if ($title === "primaryKey") {
                    $columnsString = implode(":", $field->columns);
                    $field_string .= $title . ":" . $columnsString;
                } else {
                    if ($counter != 0) {
                        $labels_string .= ",";
                    }
                    $field_string .= $title . ":" . $field->type;
                    if (!empty($field->constraints)) {
                        foreach ($field->constraints as $key => $constraint) {
                            $constraintText=null;
                            if ($key === "foreignKey"):
                                $constraint = (Object)$constraint;
                                $constraint->columns = implode(",", $constraint->columns);
                                $constraintText = ":" . "$key($constraint->tableName $constraint->columns)";
                                $relationColumns[$constraint->tableName] = $constraint->fieldColumn;
                            endif;
                            if ($key === "defaultValue"):
                                $constraintText = ":" . "$key('$constraint')";
                            endif;
                            if (empty($constraintText)&&!empty($constraint)) {
                                $constraintText = ":" . $constraint;
                            }
                            $field_string .= $constraintText;


                        }

                    }
                    $labels_string .= $title . ":" . $field->label;
                }


                $counter++;

            }

            /*die(var_dump($field_string));*/
            return ["command" => $field_string, "labels" => $labels_string, "relationsLabel" => $relationColumns];

        }

        /**
         * Returneaza atributele implicite a fiecarui clasificator
         * @return array
         */
        public static function getDefaultAttributes()
        {
            return [
                "created_at",
                "updated_at",
                "created_user",
                "updated_user",
            ];
        }

        /**
         *  * action called with POST method, destinate to create Classifier
         * -generate Migration
         * -apply Migration
         * -return paramas for action generate model
         * @param $data
         * @return string
         * @throws \yii\console\Exception
         */
        public static function ClassifierCreate($data)
        {
            $migrationName = "";
            $migrationTable = "";
            $relationColumns = [];

            if (!empty($data)) {

                $dataModel = new ClassifierForm();
                if ($dataModel->load($data)) {

                    $classifierData = (object)$data["ClassifierForm"];
                } else {
                    return "ClassifierForm don't load";
                }
            } else {
                /*data INPUT*/
                $fields = [
                    "id" => [
                        "type" => "bigInteger(10)",
                        "constraints" => [
                            //
                        ]
                    ],
                    "name" => [
                        "type" => "string(20)",
                        "constraints" => [
                            "defaultValue" => "1",
                        ]
                    ],
                    "fullName" => [
                        "type" => "string(20)",
                    ],
                    "user_id" => [
                        "type" => "bigInteger(10)",
                        "constraints" => [
                            "foreignKey" => [
                                "tableName" => "user",
                                "columns" => "username",
                                "fieldColumn" => "username"
                            ],//foreignKey=>tableName to Reference
                        ],


                    ],
                    "primaryKey" => [
                        "columns" => [
                            "id",
                            "name"
                        ]
                    ]


                ];
                $classifierData = (object)[
                    "tableName" => "test_table12",
                    "generateCrud" => false,
                    "generateTable" => true,
                    "generateModel" => true,
                    "fields" => $fields,
                    "description" => "Exemplu de descriere",
                ];
            }
            if (!$classifierData->generateTable) {
                $tableName = self::DEFAULT_SCHEMA . "." . $classifierData->tableName;
                $tableExist = Yii::$app->db->schema->getTableSchema($tableName);
                if (!$tableExist) {
                    return json_encode([
                        "error" => [
                            "attribute" => "generateTable",
                            "message" => Yii::t("classifier", "Not found table")
                        ]
                    ]);
                }

            }
            if (!$classifierData->generateModel) {
                $generator = new GeneratorHelper();

                $className = $generator->getClassName($classifierData->tableName);

                $classNameNS = CommandHelper::DEFAULT_NS . "\\" . $className;

                if (!class_exists($classNameNS)) {

                    return json_encode([
                        "error" => [
                            "attribute" => "generateModel",
                            "message" => Yii::t("classifier", "Not found model")
                        ]
                    ]);

                }

            }

            /*data INPUT end*/

            $oldApp = \Yii::$app;

            $configConsole = ArrayHelper::merge(
                require \Yii::getAlias('@common') . '/config/main.php',
                require \Yii::getAlias('@common') . '/config/main-local.php',
                require \Yii::getAlias('@console') . '/config/main.php',
                require \Yii::getAlias('@console') . '/config/main-local.php'
            );

            $configConsole["controllerMap"]["migrate"] = [
                'class' => 'classifier\controllers\MigrateController',
                'migrationTable' => 'classifier_migrations',
                "generatorTemplateFiles" => [
                    'create_table' => '@classifier/views/migration/createTableMigration.php',
                    'drop_table' => '@yii/views/dropTableMigration.php',
                    'add_column' => '@yii/views/addColumnMigration.php',
                    'drop_column' => '@yii/views/dropColumnMigration.php',
                    'create_junction' => '@yii/views/createTableMigration.php',
                ]
            ];

            $consoleApp = new Application($configConsole);

            $tableName = $classifierData->tableName;
            $classifier = SysClassifiers::findOne(["table_name" => $tableName]);
            if (empty($classifier)) {
                $classifier = new SysClassifiers();
            }
            $parsedFileds = CommandHelper::parseFields($classifierData->fields);
            $relationColumns = $parsedFileds["relationsLabel"];
            if ($classifierData->generateTable) {

                $command = $parsedFileds["command"];
                $labels = $parsedFileds["labels"];

                $migrationName = "create_" . $tableName . "_table";
                $createMigration = $consoleApp->runAction("migrate/create",
                    [
                        $migrationName,
                        'migrationPath' => '@console/migrations/' . $tableName,
                        "fields" => $command,
                        "labels" => $labels,
                        'interactive' => false,
                    ]);

                $migrate = $consoleApp->runAction("migrate/up",
                    [
                        "1",
                        'migrationPath' => '@console/migrations/' . $tableName,
                        'interactive' => false,
                    ]);

                if ($migrate !== (int)0) {
                    die(var_dump(json_encode([
                        "status" => "false",
                        "message" => "Migration failure, not up"
                    ])));

                }
                $migrationTable = $consoleApp->controllerMap["migrate"]["migrationTable"];

            }


            \Yii::$app = $oldApp;

            $classifier->table_name = $tableName;
            if ($classifierData->generateTable) {
                $classifier->last_migrate = CommandHelper::getLastMigrate($migrationName, $migrationTable);
            }
            $classifier->description = $classifierData->description;
            $classifier->attribute_fields = json_encode($classifierData->fields);
            $generator = new GeneratorHelper();
            $classifier->class_name = CommandHelper::DEFAULT_NS . "\\" . $generator->getClassName($tableName);
            if (!$classifier->save()) {
                die(var_dump($classifier->errors));
            }

            return json_encode([
                "error" => false,
                "response" => [
                    "/Classifier/default/generate-model",
                    "tableName" => $tableName,
                    "crud" => $classifierData->generateCrud,
                    "model" => $classifierData->generateModel,
                    "relationsLabel" => json_encode($relationColumns)
                ]

            ]);

        }

        /**
         * Returneaza ultima migrare pe baza de denumirea tabelului si denumirea statica a migrarii
         * @param $migrationName
         * @param $migrationTable
         * @return string
         */
        public static function getLastMigrate($migrationName, $migrationTable)
        {
            $query = (new Query())
                ->select(['version', 'apply_time'])
                ->from($migrationTable)
                ->where("version like '%$migrationName'")
                ->orderBy(['apply_time' => SORT_DESC, 'version' => SORT_DESC]);
            $migrate = $query->one();
            if (!empty($migrate["version"])) {
                return $migrate["version"];
            } else {
                return "not migrate";
            }
        }

        /**
         * Stergerea directoriiilor subdirectoriilor si fisierilor continute
         * @param $path
         * @return bool
         */
        public static function rrmdir($path)
        {
            if (is_dir($path)) {
                array_map("self::rrmdir", glob($path . DIRECTORY_SEPARATOR . '{,.[!.]}*', GLOB_BRACE));
                $result = @rmdir($path);
                if ($result == false) {
                    return $result;
                }
            } else {
                $result = @unlink($path);
                if ($result == false) {
                    return $result;
                }
            }
            return false;
        }
    }

