<?php
    /**
     * Created by PhpStorm.
     * User: startreksi
     * Date: 7/23/18
     * Time: 11:39 AM
     */

    namespace classifier\helpers;


    use Symfony\Component\Finder\Finder;
    use Yii;
    use yii\db\ActiveRecord;
    use yii\elasticsearch\Connection;
    use yii\elasticsearch\Exception;

    class CompatibilityHelper
    {
        public static function isCompatibleElastic()
        {
            if(!empty(Yii::$app->elasticsearch)) {
                $elasticExists = \Yii::$app->elasticsearch;
                try {
                    $elasticExists->getClusterState();
                } catch (\Exception $e) {

                    return false;
                }
                return true;
            }else
            {
                return false;
            }
        }

        public static function isCompatibleSNS()
        {
            if(class_exists("\classifier\helpers\SnsHelper")) {
                try {
                    $response = \classifier\helpers\SnsHelper::sendNotify("App Initialize", "Testing Connection");
                } catch (\Exception $e) {
                    return false;
                }

                return $response->getIsOk();
            }
            return false;
        }

        public static function isCompatibleDBConnection()
        {
            $elasticExists = \Yii::$app->db;
            try {
                $elasticExists->open();
            } catch (\Exception $e) {
                \Yii::$app->session->setFlash('danger',
                    "DataBase not compatible config! <br> Error: <br>" . $e->getMessage() . "<br> Trace:<br>" . $e->getTraceAsString());
                return false;
            }
            return true;
        }

        public static function isAllTableExists()
        {
            $children = array();
            $all_classes=self::get_all_fcqns(__DIR__."/../");

            foreach ($all_classes as $class) {
                if (get_parent_class($class)=="yii\db\ActiveRecord") {
                    $children[] = ["modelName"=>$class,"table"=>(new $class)::tableName(),"exists"=>!empty(Yii::$app->db->schema->getTableSchema($class::tableName()))];
                }
            }

            return $children;
        }


        static function _get_full_namespace($filename)
        {
            $lines = file_get_contents($filename);
            $fullNamespace=self::ns_by_regexp($lines);

            return $fullNamespace;
        }

        static function _get_classname($filename)
        {
            $directoriesAndFilename = explode('/', $filename);
            $filename = array_pop($directoriesAndFilename);
            $nameAndExtension = explode('.', $filename);
            $className = array_shift($nameAndExtension);

            return $className;
        }

        static function _get_filenames($path)
        {
            $finderFiles = Finder::create()->files()->in($path)->name('*.php');
            $filenames = array();
            foreach ($finderFiles as $finderFile) {
                $filenames[] = $finderFile->getRealpath();
            }

            return $filenames;
        }

        static function get_all_fcqns($path)
        {
            $filenames = self::_get_filenames($path);
            $fcqns = array();
            foreach ($filenames as $filename) {

                $fcqns[] = self::_get_full_namespace($filename);
            }

            return $fcqns;
        }
        static function ns_by_regexp($src) {
            if (preg_match('#^namespace\s+(.+?);.*class\s+(\w+).+;$#sm', $src, $m)) {
                return $m[1].'\\'.$m[2];
            }
            return null;
        }
    }