<?php
    /**
     * Created by PhpStorm.
     * User: startreksi
     * Date: 5/3/18
     * Time: 2:48 PM
     */

    namespace classifier\helpers;





    use kartik\datecontrol\DateControl;
    use kartik\number\NumberControl;
    use Yii;
    use yii\db\ActiveRecord;
    use yii\helpers\BaseHtml;
    use yii\helpers\Html;
    use yii\helpers\Inflector;
    use yii\helpers\StringHelper;
    use yii\helpers\Url;
    use yii\helpers\VarDumper;

    /**
     * Helper cu continut folosit pentru generarea automata a formelor si widge-turilor de care e nevoie
     * Class FormHelper
     * @package classifier\helpers
     */

    class FormHelper
    {

        /**
         * Generates code for active field
         * @param $model ActiveRecord
         * @param $attribute
         * @return bool
         */
        public static function  isForeignColumn($model,$attribute)
        {
            $tableSchema = $model::getTableSchema();
            foreach ($tableSchema->foreignKeys as $foreignKey)
            {

                if(!empty($foreignKey[$attribute]))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * @param $model
         * @param $modelName
         * @param $attribute
         * @param $form
         * @return mixed
         */
        public static function generateActiveField($model,$modelName,$attribute,$form)
        {
            $tableSchema = $modelName::getTableSchema();
            if ($tableSchema === false || !isset($tableSchema->columns[$attribute])) {
                if (preg_match('/^(password|pass|passwd|passcode)$/i', $attribute)) {
                    return $form->field($model, $attribute)->passwordInput();
                }

                return $form->field($model, $attribute);
            }

            if(self::isForeignColumn($modelName,$attribute))
            {
                $relationLabel=$model->relationsLabel;
                foreach ($tableSchema->foreignKeys as $foreignKey)
                {

                    if(!empty($foreignKey[$attribute]))
                    {
                        $relationTabel=$foreignKey[0];
                        $relationTabel=str_replace(CommandHelper::DEFAULT_SCHEMA.".","",$relationTabel);
                        $relationLabel=$model->relationsLabel[$relationTabel];
                        $controller=Inflector::camel2id($relationTabel);
                        break;
                    }
                }
                $attributeMethodName="getForeignData".Inflector::camel2id($attribute,'_');
                $select2= $form->field($model, $attribute)->widget('\\kartik\\widgets\\Select2', [
                    'data' => $model->$attributeMethodName(),
                    'addon' => [
                        'prepend' => [
                            'content' =>Html::button('<i class="fa fa-plus"></i>', ['id' => 'classifier_btn_sent', 'value' => \yii\helpers\Url::to(['/Classifier/'.$controller.'/create-ajax']), 'class' => 'btn btn-success open_classifier_modal']) ,
                            'asButton' => true,
                        ],
                    ],
                    'options' => ['placeholder' => Yii::t('classifier',$attribute), 'class' => 'form-control'],
                    'pluginOptions' => [
                        'allowClear' => true
                    ],
                ]);
                $AttributeName=BaseHtml::getInputId(new $modelName,$attribute);
                $script = <<< JS
        function isJson(str) {
            try {
                JSON.parse(str);
            } catch (e) {
                return false;
            }
            return true;
        }
        function initAjaxForm() {
           $('#modal_classifier form').on('submit', function (e)  {
                         e.preventDefault();
                         e.stopImmediatePropagation();
                         e.stopPropagation();
                        var form =$(this);
                        
                        jQuery.ajax({
                            url: '/Classifier/$controller/create-ajax?label=$relationLabel',
                            type: 'post',
                            complete:initAjaxForm,
                            data: form.serialize(),     
                            success: function (data) {
                              
                                if(!isJson(data))
                                    {
                                     $('#modal_classifier').modal('show')
                                            .find('.add_modal_form_classifier')
                                            .html(data)
                                             
                                 }else 
                                 {
                                     data=JSON.parse(data)
                                     var newOption = new Option( data.label,data.id, true, true);
                                     $('#$AttributeName').append(newOption).trigger('change');
                                     $('#modal_classifier').modal('hide')
                                 }
                            },
                         
                        });
                        return false;
                    });
        }
                   $("#modal_classifier").on("shown.bs.modal",function() {
                       
                     initAjaxForm();
                   })
                    $(function () {
                        $('#classifier_btn_sent').click(function () {
                            $('#modal_classifier').modal('show')
                                .find('.add_modal_form_classifier')
                                .load($(this).attr('value'))
                                
                        });
                    });
JS;

                return ["select2"=>$select2,"script"=>$script];



            }
            $column = $tableSchema->columns[$attribute];
            if ($column->phpType === 'integer') {
                return $form->field($model, $attribute)->widget(NumberControl::className(), [
                    "name" => $model,
                    'maskedInputOptions' => ['digits' => 0],
                ]);
            }
            if ($column->type === 'date') {
                return $form->field($model, $attribute)->widget(Datecontrol::className(), [
                    'type' => DateControl::FORMAT_DATE,
                    'ajaxConversion' => false,
                    'widgetOptions' => [
                        'pluginOptions' => [
                            'autoclose' => true
                        ]
                    ]
                ]);
            }
            if ($column->type === 'timestamp') {
                return $form->field($model, $attribute)->widget(Datecontrol::className(), [
                    'type' => DateControl::FORMAT_DATETIME,
                    'ajaxConversion' => false,
                    'widgetOptions' => [
                        'pluginOptions' => [
                            'autoclose' => true
                        ]
                    ]
                ]);
            }
            if ($column->type === 'time') {
                return $form->field($model, $attribute)->widget(Datecontrol::className(), [
                    'type' => DateControl::FORMAT_TIME,
                    'ajaxConversion' => false,
                    'widgetOptions' => [
                        'pluginOptions' => [
                            'autoclose' => true
                        ]
                    ]
                ]);
            }
            if ($column->phpType === 'boolean') {
                return $form->field($model, $attribute)->checkbox();
            }

            if ($column->type === 'text') {
                return $form->field($model, $attribute)->textarea(['rows' => 6]);
            }

            if (preg_match('/^(password|pass|passwd|passcode)$/i', $column->name)) {
                $input = 'passwordInput';
            } else {
                $input = 'textInput';
            }

            if (is_array($column->enumValues) && count($column->enumValues) > 0) {
                $dropDownOptions = [];
                foreach ($column->enumValues as $enumValue) {
                    $dropDownOptions[$enumValue] = Inflector::humanize($enumValue);
                }
                return $form->field($model, $attribute)->dropDownList( preg_replace("/\n\s*/", ' ', VarDumper::export($dropDownOptions)), ['prompt' => '']);
            }

            if ($column->phpType !== 'string' || $column->size === null) {
                return $form->field($model, $attribute)->$input();
            }

            return $form->field($model, $attribute)->$input(['maxlength' => true]);
        }


        /**
         * pe baza de actiune si numele la tabel returneaza url-ul spre actiune
         * @param $tableName
         * @param $action
         * @return string
         */
        public static function getClassifierUrl($className,$action)
        {
            $className=StringHelper::basename($className);
            return Url::to(["/Classifier/".Inflector::camel2id($className, '-')."/$action"]);
        }

    }