<?php

namespace classifier\models;

use classifier\helpers\CommandHelper;
use classifier\helpers\AppHelper;
use classifier\helpers\SnsHelper;
use Yii;
use yii\helpers\Inflector;
use yii\helpers\Json;
use yii\helpers\StringHelper;

/**
 * This is the model class for table "sys_classifiers".
 *
 * @property int $id
 * @property string $table_name
 * @property string $class_name
 * @property string $last_migrate
 * @property string $description
 * @property string $attributes
 * @property string $created_at
 * @property string $updated_at
 * @property int $created_user
 * @property int $updated_user
 */
class SysClassifiers extends \yii\db\ActiveRecord
{
    public function beforeValidate()
    {
        return parent::beforeValidate(); // TODO: Change the autogenerated stub
    }

    /**
     * Functie apelata inainte de salvarea clasificatorului
     * @param bool $insert
     * @return bool
     */
    public function beforeSave($insert) {

        if(empty($this->created_at)){
            $this->created_at =date("Y-m-d H:i:s");
        }
        $this->updated_at = date("Y-m-d H:i:s");
        if(empty($this->created_user))
        {
            $this->created_user=Yii::$app->user->id;
        }
        $this->updated_user=Yii::$app->user->id;

        /*SnsHelper::sendEventNotify(AppHelper::getConfigs('CLASSIFIER'), Json::encode($this->attributes));*/
        return parent::beforeSave($insert);
    }

    /**
     * returneaza numele tabelului
     * @return string
     */
    public static function tableName()
    {
        return 'sys_classifiers';
    }

    /**
     * retuneaza regulile
     * @return array
     */
    public function rules()
    {
        return [
            [['table_name', 'class_name', 'last_migrate'], 'required'],
            [['created_at', 'updated_at'], 'safe'],
            [['created_user', 'updated_user'], 'default', 'value' => null],
            [['created_user', 'updated_user'], 'integer'],
            [['table_name'], 'string', 'max' => 80],
            [['class_name', 'description'], 'string', 'max' => 500],
            [['last_migrate'], 'string', 'max' => 200],
            [['attribute_fields'], 'string', 'max' => 50000],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('classifiers', 'ID'),
            'table_name' => Yii::t('classifiers', 'Table Name'),
            'class_name' => Yii::t('classifiers', 'Class Name'),
            'last_migrate' => Yii::t('classifiers', 'Last Migrate'),
            'description' => Yii::t('classifiers', 'Description'),
            'attribute_fields' => Yii::t('classifiers', 'Attributes'),
            'created_at' => Yii::t('classifiers', 'Created At'),
            'updated_at' => Yii::t('classifiers', 'Updated At'),
            'created_user' => Yii::t('classifiers', 'Created User'),
            'updated_user' => Yii::t('classifiers', 'Updated User'),
        ];
    }
    public function crudExists()
    {

        return $this->viewsExists() && $this->controllerExists();
    }

    public function viewsExists()
    {
        $controllerId = Inflector::camel2id(StringHelper::basename($this->class_name));
        $viewPath = Yii::getAlias("@classifier") . DIRECTORY_SEPARATOR . "views" . DIRECTORY_SEPARATOR . $controllerId;
        $exists = is_dir($viewPath);
        return !empty($exists);
    }

    public function controllerExists()
    {
        $controllerClass = str_replace("models", "controllers", $this->class_name) . "Controller";
        $exists = class_exists($controllerClass);
        return $exists;
    }

    public function modelExists()
    {
        $exists = class_exists($this->class_name);
        return $exists;
    }
    public function migrationExists()
    {
        $controllerId = $this->table_name;
        $exists=is_dir(Yii::getAlias("@console").DIRECTORY_SEPARATOR."migrations".DIRECTORY_SEPARATOR.$controllerId);
        return $exists;
    }
    public function tableExists()
    {
        $table = Yii::$app->db->getTableSchema(CommandHelper::DEFAULT_SCHEMA.".".$this->table_name);
        return !empty($table);
    }
}
