<?php
declare(strict_types=1);

namespace classifier\modules\ClassifierCreator\controllers;


use classifier\helpers\CommandHelper;
use classifier\models\SysClassifiers;
use classifier\modules\ClassifierCreator\models\Classifier;
use classifier\modules\ClassifierCreator\models\ClassifierData;
use classifier\modules\ClassifierCreator\models\ClassifierField;
use kartik\form\ActiveForm;
use function var_dump;
use Yii;
use yii\console\Exception;
use yii\helpers\Json;
use yii\helpers\Url;
use yii\web\Controller;
use function json_decode;


/**
 * Class DefaultController
 * @package classifier\modules\ClassifierCreator\controllers
 */
class DefaultController extends Controller
{
    /**
     * Renders the index view for the module
     * @return string
     */
    public function actionIndex()
    {
        return $this->render('index');
    }

    /**
     *  returneaza view de baza pentru crearea clasificatorului
     * @return string
     */
    public function actionClassificatory()
    {
        $models = [new Classifier()];
        $attribute = new Classifier();
        $model = new Classifier();
        return $this->render('clasificator', ['models' => $models, 'model' => $model, 'attribute' => $attribute]);
    }
    public function actionClassifierUpdate($id)
    {
        $models = [new Classifier()];
        $attribute = new Classifier();
        $model = new Classifier();

        $data=SysClassifiers::findOne($id);
        $field_data=Json::decode($data->attribute_fields);
        $classifying = $this->getClassifying();
        return $this->render('classifier_update',
            [
                'models' => $models,
                'model' => $model,
                'attribute' => $attribute,
                "data"=>$data,
                "field_data"=>$field_data,
                "classifier"=>$classifying
            ]);
    }
    /**
     * Serializaza toate cimpurile formei de creare a clasificatorului
     * returneaza un array cu toate optiunele clasificatorului
     */
    public function actionSerialize()
    {
        $results = Yii::$app->request->post();
        $data = $results['Classifier'];


        $data_classifier = $results['classifier_data'];

        $error_array = $this->validateFormClassifier($data_classifier, $data);
        if (empty($error_array)) {
            $classifierData = (object)[];
            if (!empty($data_classifier['tableName'])) {
                $data_classifier['tableName'] = str_replace(' ', '_', $data_classifier['tableName']);
                $data_classifier['tableName'] = strtolower($data_classifier['tableName']);
                $classifierData->tableName = "cl_" . $data_classifier['tableName'];
            }
            if (!empty($data_classifier['generateCrud'])) {
                $classifierData->generateCrud = true;
            } else {
                $classifierData->generateCrud = false;
            }
            if (!empty($data_classifier['generateTable'])) {
                $classifierData->generateTable = true;
            } else {
                $classifierData->generateTable = false;
            }
            if (!empty($data_classifier['generateModel'])) {
                $classifierData->generateModel = true;
            } else {
                $classifierData->generateModel = false;

            }
            if (!empty($data_classifier['description'])) {
                $classifierData->description = $data_classifier['description'];
            } else {
                $classifierData->description = '';

            }
            $result_array = [];
            $primary_key['primaryKey'] = [];
            foreach ($data as $result) {

                $constraints = [];
                if (!empty($result['name'])) {
                    $result['name'] = str_replace(' ', '_', $result['name']);
                    $result['name'] = strtolower($result['name']);
                }
                !empty($result['null']) ? array_push($constraints, "notNull") : null;
                (!empty($result['default']) && $result['default'] !== "Default value") ? $constraints['defaultValue'] = $result['default'] : null;
//            foreignkey add

                !empty($result['option']) ?
                    !empty($result['clasificator_tip']) ? $constraints = [
                        !empty($result['null'])? "notNull": '',
                        "foreignKey" =>
                            [
                                "tableName" => $result['clasificator_tip'],
                                "columns" => $this->getForeignKey($result['clasificator_tip']),
                                "fieldColumn" => $result['option'],

                            ]
                    ]
                        : null
                    : null;
//           primaryKey
                if (!empty($result['key'])) {
                    if ($result['key'] === "primaryKey") {
                        if (!isset($primary_key['primaryKey']["columns"])) {
                            $primary_key['primaryKey'] = ["columns" => []];
                        }
                        array_push($primary_key['primaryKey']["columns"], $result['name']);
                    } elseif ($result['key']) {
                        $result['key'] === "NULL" ? null : ($constraints[] = $result['key']);
                    }
                }
                !empty($result["length"]) ? $length = "(" . $result['length'] . ")" : $length = "(" . null . ")";

                if (!empty($result['option'])) {
                    $result['type'] === "classifier" ? $type = $this->getForeignKeyType($result['clasificator_tip']) : $type = $result['type'] . $length;
                } else {
                    $type = $result['type'] . $length;
                }

                !empty($result['label']) ? $label = $result['label'] : $label = null;

                if (!empty($constraints)) {
                    $result_array[$result['name']] = [
                        'type' => $type,
                        'label' => $label,
                        'constraints' => $constraints
                    ];

                } else {
                    $result_array[$result['name']] = [
                        'type' => $type,
                        'label' => $label
                    ];
                }
            }

            if (!empty($primary_key['primaryKey'])) {
                $result_array = array_merge($result_array, $primary_key);
            }

            if (!empty($classifierData)) {
                $classifierData->fields = $result_array;
            }

            if (empty($data_classifier['update'])) {

                try {
                    $answer = CommandHelper::ClassifierCreate(["ClassifierForm" => (array)$classifierData]);
                } catch (Exception $exception) {
                    Yii::error($exception->getMessage());

                }
            }else{
                die(var_dump($classifierData));
                //todo de adugat metoda pentru prelucrare
            }


            $answerParsed = (Object)json_decode($answer);
            if ($answerParsed->error) {
                $error_array['classifier'] = [$answerParsed->error->attribute => [$answerParsed->error->message]];
                return json_encode($error_array);
            } else {

                return $this->redirect(Url::to([
                    "/Classifier/default/generate-model",
                    "tableName" => $answerParsed->response->tableName,
                    "crud" => $answerParsed->response->crud,
                    "model" => $answerParsed->response->model,
                    "relationsLabel" => $answerParsed->response->relationsLabel,
                ]));
            }


        } else {

            //   var_dump($error_array);
            return json_encode($error_array);
        }


    }

    /**
     *   Make validation of of fields of Sys Classifier
     *   if fields contains errors return a array of errors
     * @param $classifier_data
     * @param $classifier_field
     * @param int $key
     * @return array
     */
    public function validateFormClassifier($classifier_data, $classifier_field, $key = 0)
    {
        /** @var array $error_array */
        $error_array = [];
//        validate table options
        $model = new ClassifierData();
        $model->attributes = $classifier_data;
        if ($model->validate()) {

        } else {
            $error_array['classifier'] = $model->errors;
        }
        if (empty($classifier_data['update'])){
            $table_name = str_replace(' ', '_', $classifier_data['tableName']);
            $table_name = strtolower($table_name);

            if ($this->checkTableName("cl_" . $table_name) === false) {
                $error_array['classifier'] = ['tableName' => [Yii::t('index', "The table with this name already exists")]];
            }
        }

//            end validate table options
//            validate fields
        /** @var  ClassifierField $modelField */
        $modelField = new ClassifierField();

        /** @var array $unique_keys */
        $unique_keys = [];

        foreach ($classifier_field as $key => $values) {
            if (array_key_exists($values['name'], $unique_keys)) {
                $error_array['fields'] = [
                    'classifier-' . $key . '-name' => [
                        Yii::t('index', "The field name with this name already exists")
                    ]
                ];
            }
            $unique_keys[$values['name']] = $values['name'];
            $modelField->type = $values['type'];
            if (!empty($values['default'])) {
                $modelField->default = $values['default'];
            }
            if ($modelField->validate()) {
            } else {
                $error_array[$key] = $modelField->errors;
            }
        }
//            end validate fields
        /** @var array $error_array */
        return $error_array;
    }

    /**   Check if the table name exists in Data Base
     *    The parameter is string
     * @param $table
     * @return bool
     */
    public function checkTableName($table)
    {
        $connection = Yii::$app->db;//get connection
        $dbSchema = $connection->schema;
        $tables = $dbSchema->getTableSchema(CommandHelper::DEFAULT_SCHEMA . ".$table");
        if (!empty($tables)) {
            return false;
        }
        {
            return true;
        }
    }

    /**   Return the Primary Key of table
     *   table mast be string type
     * @param $table
     * @return string[]
     */
    public function getForeignKey($table)
    {
        $connection = Yii::$app->db;//get connection
        $dbSchema = $connection->schema;
        $tables = $dbSchema->getTableSchema(CommandHelper::DEFAULT_SCHEMA . ".$table");
        return $tables->primaryKey;

    }

    /**  Return the type of primary key
     * @param $table
     * @return string
     */
    public function getForeignKeyType($table)
    {
        $connection = Yii::$app->db;//get connection
        $dbSchema = $connection->schema;
        $tables = $dbSchema->getTableSchema(CommandHelper::DEFAULT_SCHEMA . ".$table");
        return $tables->getColumn($tables->primaryKey[0])->type;
    }

    /**
     * Return the view that contain the select2 input
     * the select2 options are classifiers names
     * @return string
     */
    public function actionSelect()
    {
        $classifying = $this->getClassifying();
        $key = Yii::$app->request->post("key");
        return $this->renderAjax('_form_select2', ['i' => $key, 'data_classifyng' => $classifying['classifyng']]);

    }

    /**
     * Return 2 arrays that contains the name of classifiers and as options is the
     *  columns of classifiers
     * @return array
     */
    public function getClassifying()
    {
        $connection = Yii::$app->db;//get connection
        /** @var  $dbSchema */
        $dbSchema = $connection->schema;
        $tables = $dbSchema->getTableSchemas(CommandHelper::DEFAULT_SCHEMA);//returns array of tbl schema's
        $classifying = [];
        $options = [];
        foreach ($tables as $tbl) {
            // var_dump($tbl->primaryKey);
            $classifying[$tbl->name] = $tbl->name;
            $columns = [];
            $columnNames = array_flip($tbl->getColumnNames());
            foreach ($columnNames as $key => $value) {
                $columns[$key] = $key;
            }
            $options[$tbl->name] = $columns;
        }
        return [
            'classifyng' => $classifying,
            'options' => $options
        ];
    }

    /**
     *   Return a view that contain a partial form with select2 that contains
     *   the name of columns of Classifier
     * @return string
     */
    public function actionGetOption()
    {
        $classifying = $this->getClassifying();
        $key = Yii::$app->request->post("key");
        return $this->renderAjax('_form_options', ['i' => $key, 'data_classifyng' => $classifying['classifyng']]);
    }

    /**
     *  Return the partial form that contains the
     *  options of field
     * @return string
     */
    public function actionGetSpecialOption()
    {
        $classifying = $this->getClassifying();
        $key = Yii::$app->request->post("key");
        return $this->renderAjax('_form_options_special',
            ['i' => $key, 'data_classifyng' => $classifying['classifyng']]);

    }

    /**
     * Return the partial form with text input
     *  that make input of label field
     * @return string
     */
    public function actionGetLabel()
    {
        $classifying = $this->getClassifying();
        $key = Yii::$app->request->post("key");
        return $this->renderAjax('_form_label', ['i' => $key, 'data_classifyng' => $classifying['classifyng']]);
    }

    /**
     *
     *  returneaza un input de tip select2 ce
     *  contine cimpurile clasificatorului selectat
     * @return string
     */
    public function actionSelectOption()
    {
        $data_option = $this->getClassifying();
        $key = Yii::$app->request->post("key");
        $option = Yii::$app->request->post("option");

        return $this->renderAjax('_form_select2_option',
            ['i' => $key, 'data_option' => $data_option['options'][$option]]);
    }

    /**
     *  ne creaza coloana noua in forma
     * returneaza view == _form, returneaza modelul = Calssifing
     *  returneaza indentificatorul coloanei incrementat,
     * aceasta metoda este apelata /Classifier/assets/js/js.js
     *
     *
     * @return string
     */
    public function actionItem()
    {

        $key = Yii::$app->request->post("key");
        $classifier_data = Yii::$app->request->post('classifier_data');
        $classifier_field = Yii::$app->request->post('Classifier');
        $error_array = $this->validateFormClassifier($classifier_data, $classifier_field, $key);

        if (isset($key)) {
            $model = new Classifier();
            $form = new ActiveForm();

            return $this->renderAjax('_form',
                ["form" => $form, 'model' => $model, "i" => ++$key, "error_array" => $error_array]);
        }
    }

}
